import type { AsyncFunc, AsyncResult, Params } from '@js/types';
export interface MemoAsyncOptions<T extends AsyncFunc> {
    /**
     * The function to call and cache results from
     */
    func: T;
    /**
     * Time-to-live for records in the cache. After this amount of time,
     * a value is considered "stale", and by default, will be evicted
     * from the cache.
     *
     * This value can be a number (ms), or a function which receives the
     * params and result of the call and returns a number for each record.
     */
    ttl: number | MemoAsyncTTLFunc<T>;
    /**
     * Custom function for generating a cache key. If not provided, a
     * built-in default will be used instead. (At the moment, that
     * default is `JSON.stringify()`, which is not guranteed to be
     * stable or particularly fast; Providing a better implementation
     * for most functions is recommended)
     */
    key?: MemoAsyncKeyFunc<T>;
    /**
     * Callback to run when evicting a record from the cache (usually
     * because it expired)
     */
    on_evict?: MemoAsyncOnEvict<T>;
    /**
     * If provided and non-zero, the cache will keep values in storage
     * for this long (ms) past their expiration times. During this window,
     * a subsequent request for the value will result in a new call to the
     * source to refresh the cache, but the stale value in cache will still
     * be served immediately to reduce cache misses
     */
    stale_ttl?: number;
    /**
     * If provided and non-zero, outstanding (not yet fulfilled) promises
     * will be cached and returned to subsequent callers for the given
     * amount of time (ms). If the promise does not resolve before the TTL
     * passes, it will be evicted from the cache to allow for another attempt.
     * If the promise rejects, it will be dropped from the cache. If the
     * promise resolves, the actual result will then be stored
     */
    promise_ttl?: number;
}
export interface MemoAsyncKeyFunc<T extends AsyncFunc> {
    (params: Params<T>): string;
}
export interface MemoAsyncTTLFunc<T extends AsyncFunc> {
    (params: Params<T>, result: ReturnType<T>): number;
}
export interface MemoAsyncOnEvict<T extends AsyncFunc> {
    (evicted: MemoAsyncStoredResult<T>): any;
}
export interface MemoAsyncStoredResult<T extends AsyncFunc> {
    key: string;
    created: number;
    resolved: number;
    expires: number;
    promise: ReturnType<T>;
    result: AsyncResult<T>;
    hits: number;
    timer: ReturnType<typeof setTimeout>;
}
export type MemoizedAsyncFunc<T extends AsyncFunc> = T & {
    invalidate(...params: Params<T>): void;
};
export declare function memo_async<T extends AsyncFunc>(opts: MemoAsyncOptions<T>): T;
