"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.memo_async = void 0;
function memo_async(opts) {
    const get_key = opts.key ?? default_key;
    const storage = Object.create(null);
    const refreshing = Object.create(null);
    function memoized(...params) {
        const key = get_key(params);
        if (storage[key]) {
            storage[key].hits++;
            // If the record is stale (but still stored because of `stale_ttl`), start
            // a new refresh call in the background
            if (Date.now() >= storage[key].expires) {
                refresh_stale(storage[key], params);
            }
            return storage[key].result;
        }
        const promise = opts.func(...params);
        store_record(key, params, promise);
        return promise;
    }
    function evict(record) {
        if (storage[record.key] === record) {
            delete storage[record.key];
            if (refreshing[record.key]?.timer) {
                // If another call is already working on refreshing this record and
                // it has an active `promise_ttl` timer, promote it into the main
                // storage now
                storage[record.key] = refreshing[record.key];
                delete refreshing[record.key];
            }
            if (opts.on_evict) {
                opts.on_evict(record);
            }
        }
    }
    function refresh_stale(old_record, params) {
        const record = {
            key: old_record.key,
            created: Date.now(),
            resolved: null,
            expires: null,
            promise: opts.func(...params),
            result: null,
            hits: 0,
            timer: null,
        };
        refreshing[record.key] = record;
        if (opts.promise_ttl) {
            record.timer = setTimeout(() => on_timeout(record), opts.promise_ttl);
        }
        record.promise.then((result) => on_refresh_resolve(record, params, result), (reason) => on_reject(record, reason));
    }
    function store_record(key, params, promise) {
        const record = {
            key,
            created: Date.now(),
            resolved: null,
            expires: null,
            promise,
            result: null,
            hits: 0,
            timer: null,
        };
        if (opts.promise_ttl) {
            storage[key] = record;
            record.timer = setTimeout(() => on_timeout(record), opts.promise_ttl);
        }
        promise.then((result) => on_resolve(record, params, result), (reason) => on_reject(record, reason));
    }
    function on_timeout(record) {
        record.timer = null;
        if (storage[record.key] === record) {
            delete storage[record.key];
        }
    }
    function on_resolve(record, params, result) {
        if (record.timer) {
            clearTimeout(record.timer);
        }
        const ttl = typeof opts.ttl === 'function'
            ? opts.ttl(params, result)
            : opts.ttl;
        const evict_after = opts.stale_ttl
            ? ttl + opts.stale_ttl
            : ttl;
        record.result = result;
        record.resolved = Date.now();
        record.expires = record.resolved + ttl;
        record.timer = setTimeout(() => evict(record), evict_after);
        storage[record.key] = record;
    }
    function on_refresh_resolve(record, params, result) {
        if (record.timer) {
            clearTimeout(record.timer);
        }
        const ttl = typeof opts.ttl === 'function'
            ? opts.ttl(params, result)
            : opts.ttl;
        const evict_after = opts.stale_ttl
            ? ttl + opts.stale_ttl
            : ttl;
        record.result = result;
        record.resolved = Date.now();
        record.expires = record.resolved + ttl;
        record.timer = setTimeout(() => evict(record), evict_after);
        if (storage[record.key]?.timer) {
            clearTimeout(storage[record.key].timer);
            evict(storage[record.key]);
        }
        storage[record.key] = record;
    }
    function on_reject(record, reason) {
        if (record.timer) {
            // If a timeout timer was set due to `promise_ttl`, clear it now
            clearTimeout(record.timer);
        }
        if (storage[record.key] === record) {
            delete storage[record.key];
        }
    }
    memoized.invalidate = function invalidate(...params) {
        const key = get_key(params);
        if (storage[key]) {
            evict(storage[key]);
        }
    };
    return memoized;
}
exports.memo_async = memo_async;
function default_key(...params) {
    return JSON.stringify(params);
}
//# sourceMappingURL=async.js.map