import type { Func, Params } from '@js/types';
export interface MemoOptions<T extends Func> {
    /**
     * The function to call and cache results from
     */
    func: T;
    /**
     * Time-to-live for records in the cache. After this amount of time,
     * a value is considered "stale", and by default, will be evicted
     * from the cache.
     *
     * This value can be a number (ms), or a function which receives the
     * params and result of the call and returns a number for each record.
     */
    ttl: number | MemoTTLFunc<T>;
    /**
     * Custom function for generating a cache key. If not provided, a
     * built-in default will be used instead. (At the moment, that
     * default is `JSON.stringify()`, which is not guranteed to be
     * stable or particularly fast; Providing a better implementation
     * for most functions is recommended)
     */
    key?: MemoKeyFunc<T>;
    /**
     * Callback to run when evicting a record from the cache (usually
     * because it expired)
     */
    on_evict?: MemoOnEvict<T>;
}
export interface MemoKeyFunc<T extends Func> {
    (params: Params<T>): string;
}
export interface MemoTTLFunc<T extends Func> {
    (params: Params<T>, result: ReturnType<T>): number;
}
export interface MemoOnEvict<T extends Func> {
    (evicted: MemoStoredResult<T>): any;
}
export interface MemoStoredResult<T extends Func> {
    key: string;
    created: number;
    expires: number;
    result: ReturnType<T>;
    hits: number;
    timer: ReturnType<typeof setTimeout>;
}
export type MemoizedFunc<T extends Func> = T & {
    invalidate(...params: Params<T>): void;
};
export declare function memo<T extends Func>(opts: MemoOptions<T>): MemoizedFunc<T>;
